@@ -9,6 +9,7 @@
 #include "surface.h"
 #include "dbus.h"
 #include "mako.h"
+#include "mode.h"
 #include "notification.h"
 #include "wayland.h"
 
@@ -312,8 +313,7 @@ static int handle_set_mode(sd_bus_message *msg, void *data,
 		return ret;
 	}
 
-	free(state->current_mode);
-	state->current_mode = strdup(mode);
+	set_modes(state, &mode, 1);
 
 	reapply_config(state);
 
@@ -336,6 +336,83 @@ static int handle_reload(sd_bus_message *msg, void *data,
 	return sd_bus_reply_method_return(msg, "");
 }
 
+static int handle_list_modes(sd_bus_message *msg, void *data,
+		sd_bus_error *ret_error) {
+	struct mako_state *state = data;
+
+	sd_bus_message *reply = NULL;
+	int ret = sd_bus_message_new_method_return(msg, &reply);
+	if (ret < 0) {
+		return ret;
+	}
+
+	ret = sd_bus_message_open_container(reply, 'a', "s");
+	if (ret < 0) {
+		return ret;
+	}
+
+	const char **mode_ptr;
+	wl_array_for_each(mode_ptr, &state->current_modes) {
+		ret = sd_bus_message_append_basic(reply, 's', *mode_ptr);
+		if (ret < 0) {
+			return ret;
+		}
+	}
+
+	ret = sd_bus_message_close_container(reply);
+	if (ret < 0) {
+		return ret;
+	}
+
+	ret = sd_bus_send(NULL, reply, NULL);
+	if (ret < 0) {
+		return ret;
+	}
+
+	sd_bus_message_unref(reply);
+	return 0;
+}
+
+static int handle_set_modes(sd_bus_message *msg, void *data,
+		sd_bus_error *ret_error) {
+	struct mako_state *state = data;
+
+	int ret = sd_bus_message_enter_container(msg, 'a', "s");
+	if (ret < 0) {
+		return ret;
+	}
+
+	struct wl_array modes_arr;
+	wl_array_init(&modes_arr);
+	while (true) {
+		const char *mode;
+		ret = sd_bus_message_read(msg, "s", &mode);
+		if (ret < 0) {
+			return ret;
+		} else if (ret == 0) {
+			break;
+		}
+
+		const char **dst = wl_array_add(&modes_arr, sizeof(char *));
+		*dst = mode;
+	}
+
+	ret = sd_bus_message_exit_container(msg);
+	if (ret < 0) {
+		return ret;
+	}
+
+	const char **modes = modes_arr.data;
+	size_t modes_len = modes_arr.size / sizeof(char *);
+	set_modes(state, modes, modes_len);
+
+	wl_array_release(&modes_arr);
+
+	reapply_config(state);
+
+	return sd_bus_reply_method_return(msg, "");
+}
+
 static const sd_bus_vtable service_vtable[] = {
 	SD_BUS_VTABLE_START(0),
 	SD_BUS_METHOD("DismissAllNotifications", "", "", handle_dismiss_all_notifications, SD_BUS_VTABLE_UNPRIVILEGED),
@@ -347,6 +424,8 @@ static const sd_bus_vtable service_vtable[] = {
 	SD_BUS_METHOD("ListNotifications", "", "aa{sv}", handle_list_notifications, SD_BUS_VTABLE_UNPRIVILEGED),
 	SD_BUS_METHOD("Reload", "", "", handle_reload, SD_BUS_VTABLE_UNPRIVILEGED),
 	SD_BUS_METHOD("SetMode", "s", "", handle_set_mode, SD_BUS_VTABLE_UNPRIVILEGED),
+	SD_BUS_METHOD("ListModes", "", "as", handle_list_modes, SD_BUS_VTABLE_UNPRIVILEGED),
+	SD_BUS_METHOD("SetModes", "as", "", handle_set_modes, SD_BUS_VTABLE_UNPRIVILEGED),
 	SD_BUS_VTABLE_END
 };
 
