@@ -7,6 +7,7 @@
 #define MAX_TOUCHPOINTS 10
 
 struct mako_state;
+struct mako_surface;
 
 struct mako_output {
 	struct mako_state *state;
@@ -28,18 +29,22 @@ struct mako_seat {
 	struct {
 		struct wl_pointer *wl_pointer;
 		int32_t x, y;
+		struct mako_surface *surface;
 	} pointer;
 
 	struct {
 		struct wl_touch *wl_touch;
 		struct {
 			int32_t x, y;
+			struct mako_surface *surface;
 		} pts[MAX_TOUCHPOINTS];
 	} touch;
 };
 
 bool init_wayland(struct mako_state *state);
 void finish_wayland(struct mako_state *state);
-void set_dirty(struct mako_state *state);
+void set_dirty(struct mako_surface *surface);
+char *create_xdg_activation_token(struct mako_surface *surface,
+	struct mako_seat *seat, uint32_t serial);
 
 #endif
