@@ -7,8 +7,10 @@
 #include "config.h"
 #include "dbus.h"
 #include "mako.h"
+#include "mode.h"
 #include "notification.h"
 #include "render.h"
+#include "surface.h"
 #include "wayland.h"
 
 static const char usage[] =
@@ -21,6 +23,9 @@ static const char usage[] =
 	"      --text-color <color>            Text color.\n"
 	"      --width <px>                    Notification width.\n"
 	"      --height <px>                   Max notification height.\n"
+	"      --outer-margin <px>[,<px>...]   Outer margin values, comma separated.\n"
+	"                                      Up to four values, with the same\n"
+	"                                      meaning as in CSS.\n"
 	"      --margin <px>[,<px>...]         Margin values, comma separated.\n"
 	"                                      Up to four values, with the same\n"
 	"                                      meaning as in CSS.\n"
@@ -40,6 +45,8 @@ static const char usage[] =
 	"      --format <format>               Format string.\n"
 	"      --hidden-format <format>        Format string.\n"
 	"      --max-visible <n>               Max number of visible notifications.\n"
+	"      --max-history <n>               Max size of history buffer.\n"
+	"      --history <0|1>                 Add expired notifications to history.\n"
 	"      --sort <sort_criteria>          Sorts incoming notifications by time\n"
 	"                                      and/or priority in ascending(+) or\n"
 	"                                      descending(-) order.\n"
@@ -65,14 +72,32 @@ static bool init(struct mako_state *state) {
 		return false;
 	}
 	wl_list_init(&state->notifications);
+	wl_list_init(&state->history);
+	wl_array_init(&state->current_modes);
+	const char *mode = "default";
+	set_modes(state, &mode, 1);
 	return true;
 }
 
 static void finish(struct mako_state *state) {
+	char **mode_ptr;
+	wl_array_for_each(mode_ptr, &state->current_modes) {
+		free(*mode_ptr);
+	}
+	wl_array_release(&state->current_modes);
+
 	struct mako_notification *notif, *tmp;
 	wl_list_for_each_safe(notif, tmp, &state->notifications, link) {
 		destroy_notification(notif);
 	}
+	wl_list_for_each_safe(notif, tmp, &state->history, link) {
+		destroy_notification(notif);
+	}
+
+	struct mako_surface *surface, *stmp;
+	wl_list_for_each_safe(surface, stmp, &state->surfaces, link) {
+		destroy_surface(surface);
+	}
 	finish_event_loop(&state->event_loop);
 	finish_wayland(state);
 	finish_dbus(state);
@@ -86,13 +111,17 @@ int main(int argc, char *argv[]) {
 	state.argc = argc;
 	state.argv = argv;
 
+	wl_list_init(&state.surfaces);
+
 	// This is a bit wasteful, but easier than special-casing the reload.
 	init_default_config(&state.config);
 	int ret = reload_config(&state.config, argc, argv);
 
 	if (ret < 0) {
+		finish_config(&state.config);
 		return EXIT_FAILURE;
 	} else if (ret > 0) {
+		finish_config(&state.config);
 		printf("%s", usage);
 		return EXIT_SUCCESS;
 	}
