@@ -94,6 +94,15 @@ void destroy_notification(struct mako_notification *notif) {
 	wl_list_remove(&notif->link);
 
 	reset_notification(notif);
+
+	free(notif->app_name);
+	free(notif->app_icon);
+	free(notif->summary);
+	free(notif->body);
+	free(notif->category);
+	free(notif->desktop_entry);
+	free(notif->tag);
+
 	finish_style(&notif->style);
 	free(notif);
 }
@@ -256,6 +265,8 @@ char *format_notif_text(char variable, bool *markup, void *data) {
 	switch (variable) {
 	case 'a':
 		return strdup(notif->app_name);
+	case 'i':
+		return mako_asprintf("%d", notif->id);
 	case 's':
 		return strdup(notif->summary);
 	case 'b':
