@@ -1,5 +1,6 @@
 #define _POSIX_C_SOURCE 200809L
 #include <assert.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -63,9 +64,13 @@ static const struct wl_output_listener output_listener = {
 	.scale = output_handle_scale,
 };
 
+static void send_frame(struct mako_surface *surface);
+
 static void create_output(struct mako_state *state,
 		struct wl_output *wl_output, uint32_t global_name) {
 	struct mako_output *output = calloc(1, sizeof(struct mako_output));
+	struct mako_surface *surface;
+	bool recreate_surface = false;
 	if (output == NULL) {
 		fprintf(stderr, "allocation failed\n");
 		return;
@@ -74,11 +79,20 @@ static void create_output(struct mako_state *state,
 	output->global_name = global_name;
 	output->wl_output = wl_output;
 	output->scale = 1;
+
+	recreate_surface = wl_list_empty(&state->outputs);
+
 	wl_list_insert(&state->outputs, &output->link);
 
 	wl_output_set_user_data(wl_output, output);
 	wl_output_add_listener(wl_output, &output_listener, output);
 	get_xdg_output(output);
+	if (recreate_surface) {
+		// We had no outputs, force our surfaces to redraw
+		wl_list_for_each(surface, &output->state->surfaces, link) {
+			set_dirty(surface);
+		}
+	}
 }
 
 static void destroy_output(struct mako_output *output) {
@@ -162,6 +176,45 @@ static void touch_handle_up(void *data, struct wl_touch *wl_touch,
 	seat->touch.pts[id].surface = NULL;
 }
 
+static void load_cursor(struct mako_state *state, uint32_t scale) {
+	const char *cursor_name = "left_ptr";
+
+	//don't reload the cursor if what we have already can be used
+	if (state->cursor.theme != NULL && state->cursor.scale == scale) {
+		return;
+	}
+
+	if (state->cursor.theme != NULL) {
+		wl_cursor_theme_destroy(state->cursor.theme);
+	}
+
+	if (state->cursor.surface == NULL) {
+		state->cursor.surface = wl_compositor_create_surface(state->compositor);
+	}
+
+	const char *xcursor_theme = getenv("XCURSOR_THEME");
+	state->cursor.theme = wl_cursor_theme_load(xcursor_theme, state->cursor.size * scale, state->shm);
+	if (state->cursor.theme == NULL) {
+		fprintf(stderr, "couldn't find a cursor theme\n");
+		return;
+	}
+	struct wl_cursor *cursor = wl_cursor_theme_get_cursor(state->cursor.theme, cursor_name);
+	if (cursor == NULL) {
+		fprintf(stderr, "couldn't find cursor icon \"%s\"\n", cursor_name);
+		wl_cursor_theme_destroy(state->cursor.theme);
+		// Set to NULL so it doesn't get free'd again
+		state->cursor.theme = NULL;
+		return;
+	}
+
+	state->cursor.scale = scale;
+	state->cursor.image = cursor->images[0];
+	struct wl_buffer *cursor_buffer = wl_cursor_image_get_buffer(cursor->images[0]);
+	wl_surface_attach(state->cursor.surface, cursor_buffer, 0, 0);
+	wl_surface_set_buffer_scale(state->cursor.surface, scale);
+	wl_surface_commit(state->cursor.surface);
+}
+
 static void pointer_handle_enter(void *data, struct wl_pointer *wl_pointer,
 		uint32_t serial, struct wl_surface *wl_surface,
 		wl_fixed_t surface_x, wl_fixed_t surface_y) {
@@ -172,10 +225,22 @@ static void pointer_handle_enter(void *data, struct wl_pointer *wl_pointer,
 	seat->pointer.y = wl_fixed_to_int(surface_y);
 	seat->pointer.surface = get_surface(state, wl_surface);
 
+	int scale = 1;
+
+	struct mako_surface *surface;
+	wl_list_for_each(surface, &state->surfaces, link) {
+		if (surface->surface_output->scale > scale) {
+			scale = surface->surface_output->scale;
+		}
+	}
+
 	// Change the mouse cursor to "left_ptr"
-	if (state->cursor_theme != NULL) {
-		wl_pointer_set_cursor(wl_pointer, serial, state->cursor_surface,
-			state->cursor_image->hotspot_x, state->cursor_image->hotspot_y);
+	load_cursor(state, scale);
+
+	if (state->cursor.theme != NULL) {
+		wl_pointer_set_cursor(wl_pointer, serial, state->cursor.surface,
+			state->cursor.image->hotspot_x / state->cursor.scale,
+			state->cursor.image->hotspot_y / state->cursor.scale);
 	}
 }
 
@@ -305,18 +370,25 @@ static const struct wl_surface_listener surface_listener = {
 
 
 static void schedule_frame_and_commit(struct mako_surface *state);
-static void send_frame(struct mako_surface *surface);
 
 static void layer_surface_handle_configure(void *data,
 		struct zwlr_layer_surface_v1 *surface,
 		uint32_t serial, uint32_t width, uint32_t height) {
 	struct mako_surface *msurface = data;
 
+	zwlr_layer_surface_v1_ack_configure(surface, serial);
+
+	if (msurface->configured &&
+			msurface->width == (int32_t) width &&
+			msurface->height == (int32_t) height) {
+		wl_surface_commit(msurface->surface);
+		return;
+	}
+
 	msurface->configured = true;
 	msurface->width = width;
 	msurface->height = height;
 
-	zwlr_layer_surface_v1_ack_configure(surface, serial);
 	send_frame(msurface);
 }
 
@@ -465,24 +537,8 @@ bool init_wayland(struct mako_state *state) {
 			fprintf(stderr, "Error: XCURSOR_SIZE is invalid\n");
 		}
 	}
-	state->cursor_theme = wl_cursor_theme_load(NULL, cursor_size, state->shm);
-	if (state->cursor_theme == NULL) {
-		fprintf(stderr, "couldn't find a cursor theme\n");
-		return true;
-	}
-	struct wl_cursor *cursor = wl_cursor_theme_get_cursor(state->cursor_theme, "left_ptr");
-	if (cursor == NULL) {
-		fprintf(stderr, "couldn't find cursor icon \"left_ptr\"\n");
-		wl_cursor_theme_destroy(state->cursor_theme);
-		// Set to NULL so it doesn't get free'd again
-		state->cursor_theme = NULL;
-		return true;
-	}
-	state->cursor_image = cursor->images[0];
-	struct wl_buffer *cursor_buffer = wl_cursor_image_get_buffer(cursor->images[0]);
-	state->cursor_surface = wl_compositor_create_surface(state->compositor);
-	wl_surface_attach(state->cursor_surface, cursor_buffer, 0, 0);
-	wl_surface_commit(state->cursor_surface);
+
+	state->cursor.size = cursor_size;
 
 	return true;
 }
@@ -510,9 +566,9 @@ void finish_wayland(struct mako_state *state) {
 		zxdg_output_manager_v1_destroy(state->xdg_output_manager);
 	}
 
-	if (state->cursor_theme != NULL) {
-		wl_cursor_theme_destroy(state->cursor_theme);
-		wl_surface_destroy(state->cursor_surface);
+	if (state->cursor.theme != NULL) {
+		wl_cursor_theme_destroy(state->cursor.theme);
+		wl_surface_destroy(state->cursor.surface);
 	}
 
 	zwlr_layer_shell_v1_destroy(state->layer_shell);
@@ -560,6 +616,11 @@ static void schedule_frame_and_commit(struct mako_surface *surface);
 static void send_frame(struct mako_surface *surface) {
 	struct mako_state *state = surface->state;
 
+	if (wl_list_empty(&state->outputs)) {
+		surface->dirty = false;
+		return;
+	}
+
 	int scale = 1;
 	if (surface->surface_output != NULL) {
 		scale = surface->surface_output->scale;
@@ -642,6 +703,9 @@ static void send_frame(struct mako_surface *surface) {
 				height);
 		zwlr_layer_surface_v1_set_anchor(surface->layer_surface,
 				surface->anchor);
+		zwlr_layer_surface_v1_set_margin(surface->layer_surface,
+			style->outer_margin.top, style->outer_margin.right,
+			style->outer_margin.bottom, style->outer_margin.left);
 		wl_surface_commit(surface->surface);
 
 		// Now we're going to bail without drawing anything. This gives the
